--[[
	Title: Player

	Holds some helpful player functions.
]]

--[[
	Table: slapSounds

	These are the sounds used for slaps.
]]
local slapSounds = {
	"physics/body/body_medium_impact_hard1.wav",
	"physics/body/body_medium_impact_hard2.wav",
	"physics/body/body_medium_impact_hard3.wav",
	"physics/body/body_medium_impact_hard5.wav",
	"physics/body/body_medium_impact_hard6.wav",
	"physics/body/body_medium_impact_soft5.wav",
	"physics/body/body_medium_impact_soft6.wav",
	"physics/body/body_medium_impact_soft7.wav",
}


--[[
	Function: slap

	Slaps an entity, can be a user or any entity.

	Parameters:

		ent - The target ent.
		damage - *(Optional, defaults to 0)* The amount of damage to inflict on the entity.
		power - *(Optional, defaults to 30)* The power of the slap.
		nosound - *(Optional, defaults to false)* If true, no sound will be played.
]]
function ULib.slap( ent, damage, power, nosound )
	if ent:GetMoveType() == MOVETYPE_OBSERVER then return end -- Nothing we can do.

	damage = damage or 0
	power = power or 500

	if ent:IsPlayer() then
		if not ent:Alive() then 
			return -- Nothing we can do.
		end

		if ent:InVehicle() then 
			ent:ExitVehicle() 
		end

		if ent:GetMoveType() == MOVETYPE_NOCLIP then
			ent:SetMoveType( MOVETYPE_WALK )
		end
	end

	if not nosound then -- Play a slap sound
		local sound_num = math.random( #slapSounds ) -- Choose at random
		ent:EmitSound( slapSounds[ sound_num ] )
	end

	local direction = Vector( math.random( 20 )-10, math.random( 20 )-10, math.random( 20 )-5 ) -- Make it random, slightly biased to go up.
	ULib.applyAccel( ent, power, direction )

	local newHp = ent:Health() - damage
	if newHp <= 0 then
		if ent:IsPlayer() then
			ent:Kill()
		else
			ent:Fire( "break", 1, 0 )
		end
		return
	end
	ent:SetHealth( newHp )
end

--[[
	Function: kick

	Kicks a user.

	Parameters:

		ply - The player to kick.
		reason - *(Optional)* The reason to give for kicking.
]]
function ULib.kick( ply, reason )
	if reason and type( reason ) == "string" then
		game.ConsoleCommand( "kickid " .. ply:UserID() .. " \"" .. reason .. "\"\n" )
	else
		game.ConsoleCommand( "kickid " .. ply:UserID() .. "\n" )
	end
end


--[[
	Function: ban

	Bans a user.

	Parameters:

		ply - The player to ban.
		time - *(Optional)* The time in minutes to ban the person for, leave nil or 0 for permaban.
]]
function ULib.ban( ply, time )
	if not time or type( time ) ~= "number" then
		time = 0
	end

	-- Load our currently banned users so we don't overwrite them
	if file.Exists( "../cfg/banned_user.cfg" ) then
		game.ConsoleCommand( "exec banned_user.cfg\n" )
	end

	game.ConsoleCommand( "banid " .. time .. " " .. ply:UserID() .. ";writeid\n" )
end


--[[
	Function: kickban

	Kicks and bans a user.

	Parameters:

		ply - The player to ban.
		time - *(Optional)* The time in minutes to ban the person for, leave nil or 0 for permaban.
]]
function ULib.kickban( ply, time )
	if not time or type( time ) ~= "number" then
		time = 0
	end
	
	-- Load our currently banned users so we don't overwrite them
	if file.Exists( "../cfg/banned_user.cfg" ) then
		game.ConsoleCommand( "exec banned_user.cfg\n" )
	end

	game.ConsoleCommand( "banid " .. time .. " " .. ply:UserID() .. " kick;writeid\n" )
end

local function doInvis()
	local players = player.GetAll()
	local remove = true
	for _, player in ipairs( players ) do
		local t = player:GetTable()
		if t.invis then
			remove = false
			if player:Alive() and player:GetActiveWeapon():IsValid() then
				if player:GetActiveWeapon() ~= t.invis.wep then
					timer.Simple( 0.05, ULib.invisible, player, true, t.invis.vis )
					t.invis.wep = player:GetActiveWeapon()
				end
			end
		end
	end
end

--[[
	Function: invisible

	Makes a user invisible

	Parameters:

		ply - The player to affect.
		bool - Whether they're invisible or not
		visibility - *(Optional, defaults to 0)* A number from 0 to 255 for their visibility.
]]
function ULib.invisible( ply, bool, visibility )
	if bool then
		visibility = visibility or 0
		ply:SetRenderMode( RENDERMODE_TRANSALPHA )
		ply:Fire( "alpha", visibility, 0 )
		ply:GetTable().invis = { vis=visibility, wep=ply:GetActiveWeapon() }

		if ply:GetActiveWeapon():IsValid() then
			ply:GetActiveWeapon():SetRenderMode( RENDERMODE_TRANSALPHA )
			ply:GetActiveWeapon():Fire( "alpha", visibility, 0 )
			if ply:GetActiveWeapon():GetClass() == "gmod_tool" then
				ply:DrawWorldModel( false ) -- tool gun has problems
			else
				ply:DrawWorldModel( true )
			end
		end

		hook.Add( "Think", "InvisThink", doInvis )
	else
		ply:SetRenderMode( RENDERMODE_NORMAL )
		ply:Fire( "alpha", 255, 0 )
		ply:GetActiveWeapon():SetRenderMode( RENDERMODE_NORMAL )
		ply:GetActiveWeapon():Fire( "alpha", 255, 0 )
		ply:GetTable().invis = nil
	end
end